//
//  MSCampaignsViewController.m
//  MeridianSamples
//
//  Copyright © 2015 Aruba Networks. All rights reserved.
//


#import "MSCampaignsViewController.h"
#import <UserNotifications/UserNotifications.h>

// For NSUserDefaults
NSString *MSCampaignsViewControllerShouldBeMonitoringKey = @"MSCampaignsViewControllerShouldBeMonitoring";

@interface MSCampaignsViewController () <MRCampaignManagerDelegate>
@property (nonatomic, copy) MREditorKey *appKey;
@property (nonatomic, strong) MRCampaignManager *campaignManager;
@property (nonatomic) BOOL shouldBeMonitoring;
@end

@implementation MSCampaignsViewController

- (instancetype)init {
    if ((self = [super initWithStyle:UITableViewStyleGrouped])) {
        self.edgesForExtendedLayout = UIRectEdgeNone;
    }
    return self;
}

- (void)dealloc {

    // shut down our campaign manager if it was running
    [self.campaignManager stopMonitoring];
    self.shouldBeMonitoring = NO;

    // unregister for notifications
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

#pragma mark - UIViewController

- (void)loadView {
    [super loadView];

    self.campaignManager = [[MRCampaignManager alloc] initWithApp:[MREditorKey keyWithIdentifier:[MSExampleHost appID]]];
    self.campaignManager.delegate = self;

    // sign up to know when the application goes to foreground so we can clear any notifications from Notification Center
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillEnterForeground) name:UIApplicationWillEnterForegroundNotification object:nil];

    // resume monitoring if necessary
    if (self.shouldBeMonitoring)
        [self.campaignManager startMonitoring];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];

    // let the user know that leaving while monitoring will cancel monitoring.
    if (self.isMovingFromParentViewController && self.shouldBeMonitoring) {
        UIAlertController *alertController =
        [UIAlertController alertControllerWithTitle:@"Campaign Monitoring"
                                            message:@"Campaign Monitoring has been stopped."
                                     preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];
    }
}

#pragma mark - MSExampleViewController

+ (NSString *)exampleTitle {
    return @"Campaigns";
}

+ (NSString *)exampleInfo {
    return @"Example of how to use the MRCampaignManager API to listen for proximity beacons and trigger campaigns, as directed by the Meridian Editor.";
}

#pragma mark - UITableView

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (section == 0) {
        return 1;
    }
    else if (section == 1) {
        return 1;
    }
    else {
        return 0;
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {

    UITableViewCell *cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:nil];
    cell.textLabel.font = [UIFont systemFontOfSize:20];

    if (indexPath.section == 0) {
        cell.textLabel.text = self.shouldBeMonitoring ? @"Stop Monitoring" : @"Start Monitoring";
    }
    else if (indexPath.section == 1) {
        cell.textLabel.text = @"Reset Campaigns";
    }

    return cell;
}

- (NSString *)tableView:(UITableView *)tableView titleForFooterInSection:(NSInteger)section {
    if (section == 0) {
        return @"Starts or stops background monitoring for Campaigns. When started, Campaigns will be monitored in the background.\n\nIMPORTANT: Campaign triggers will only occur when this page is being displayed. You can leave the app or turn your screen off, but if the app is shut down, you will need to navigate back to this screen for monitoring to continue.";
    }
    else if (section == 1) {
        return @"Resets the Campaign 'cooldown' so that Campaign triggers can create notifications immediately instead of waiting for a set period (normally around 12 hours)";
    }
    else {
        return nil;
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (indexPath.section == 0) {

        // toggle monitoring
        if (!self.shouldBeMonitoring) {

            if (TARGET_IPHONE_SIMULATOR) {
                UIAlertController *alertController =
                [UIAlertController alertControllerWithTitle:@"Campaign Monitoring"
                                                    message:@"Campaign Monitoring is not supported in the simulator."
                                             preferredStyle:UIAlertControllerStyleAlert];
                [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
                [self presentViewController:alertController animated:YES completion:nil];
            }
            else {

                // register to display local notifications
                [[UNUserNotificationCenter currentNotificationCenter]
                 requestAuthorizationWithOptions:UNAuthorizationOptionBadge|UNAuthorizationOptionAlert|UNAuthorizationOptionSound
                 completionHandler:^(BOOL granted, NSError * _Nullable error) {
                     // Nothing to see here
                 }];

                [self.campaignManager startMonitoring];
                self.shouldBeMonitoring = YES;
            }
        }
        else {
            [self.campaignManager stopMonitoring];
            self.shouldBeMonitoring = NO;
        }

        // update the "Start/Stop Monitoring" text
        [self.tableView reloadData];
    }
    else if (indexPath.section == 1) {
        [self.campaignManager resetAllCampaignsWithCompletionHandler:^(NSError * _Nullable error) {

            if (!error) {
                UIAlertController *alertController =
                [UIAlertController alertControllerWithTitle:@"Campaign Reset Complete"
                                                    message:nil
                                             preferredStyle:UIAlertControllerStyleAlert];
                [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
                [self presentViewController:alertController animated:YES completion:nil];
            }
            else {
                UIAlertController *alertController =
                [UIAlertController alertControllerWithTitle:@"Campaign Reset Error"
                                                    message:error.localizedDescription
                                             preferredStyle:UIAlertControllerStyleAlert];
                [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
                [self presentViewController:alertController animated:YES completion:nil];
            }
        }];
    }

    // deselect whatever cell was selected
    [self.tableView deselectRowAtIndexPath:indexPath animated:YES];
}

#pragma mark - State Persisting

// We need to remember whether we were monitoring for campaigns previously in case we were relaunched - that way we can
// resume monitoring which has the side-effect of triggering any campaigns that were waiting to be triggered on relaunch.

- (BOOL)shouldBeMonitoring {
    return [[NSUserDefaults standardUserDefaults] boolForKey:MSCampaignsViewControllerShouldBeMonitoringKey];
}

- (void)setShouldBeMonitoring:(BOOL)monitor {
    [[NSUserDefaults standardUserDefaults] setBool:monitor forKey:MSCampaignsViewControllerShouldBeMonitoringKey];
}

#pragma mark - MRCampaignManagerDelegate

- (BOOL)campaignManager:(MRCampaignManager *)manager shouldGenerateNotificationForCampaign:(NSDictionary *)campaignInfo {

    NSLog(@"About to trigger campaign with title '%@'", campaignInfo[@"title"]);

    // We could return NO or create our own UIAlert or NSNotification based on the contents of campaignInfo if we wished.
    // Returning YES will cause the default behavior, typically a local notification.

    // Since we don't want to pollute our app delegate with logic to generate an alert if we're foregrounded, we'll handle
    // that case here.
    if ([UIApplication sharedApplication].applicationState == UIApplicationStateActive) {

        UIAlertController *alertController =
        [UIAlertController alertControllerWithTitle:@"Campaign Triggered"
                                            message:campaignInfo[@"message"]
                                     preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
        [self presentViewController:alertController animated:YES completion:nil];

        // we handled it so MRCampaignManager doesn't have to.
        return NO;
    }
    else {
        // allow MRCampaignManager to handle it the usual way (posting a local notification)
        return YES;
    }
}

- (void)applicationWillEnterForeground {
    // clear out any notifications generated by MRCampaignManager
    [[UNUserNotificationCenter currentNotificationCenter] removeAllPendingNotificationRequests];
}

- (void)campaignManager:(MRCampaignManager *)manager didFailWithError:(NSError *)error {

    NSLog(@"Campaign manager failed with error: %@", error);

    UIAlertController *alertController =
    [UIAlertController alertControllerWithTitle:@"Campaign Error"
                                        message:error.localizedDescription
                                 preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alertController animated:YES completion:nil];
}

@end
