//
//  MSAppleMapViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 7/3/19.
//  Copyright © 2019 Aruba Networks. All rights reserved.
//

import UIKit
import MapKit

private extension Selector {
    static let zoomTapped = #selector(MSAppleMapViewController.zoomTapped(_:))
}

class MSAppleMapViewController: UIViewController {
    var appleMapView = MKMapView()
    var meridianMap: MRMap?
    var meridianMapOverlay: MKPolygon?
    var latitudeSpanTrigger: CLLocationDegrees = 0.0
    var longitudeSpanTrigger: CLLocationDegrees = 0.0

    override func viewDidLoad() {
        super.viewDidLoad()
        
        view.addSubview(appleMapView)
        appleMapView.translatesAutoresizingMaskIntoConstraints = false
        appleMapView.topAnchor.constraint(equalTo: view.topAnchor).isActive = true
        appleMapView.bottomAnchor.constraint(equalTo: view.bottomAnchor).isActive = true
        appleMapView.leadingAnchor.constraint(equalTo: view.leadingAnchor).isActive = true
        appleMapView.trailingAnchor.constraint(equalTo: view.trailingAnchor).isActive = true
        appleMapView.delegate = self
        
        let button0 = UIBarButtonItem(title: "Zoom", style: .plain, target: self, action: .zoomTapped)
        navigationItem.rightBarButtonItems = [button0]

        MRMap.getMap(MREditorKey(forMap: MSExampleHost.mapID(), app: MSExampleHost.appID()), success: { (map) in
            self.meridianMap = map

            // Create the apple map overlay
            let size = map.size
            let topLeft     = map.gps(withMapPt: .zero)
            let topRight    = map.gps(withMapPt: CGPoint(x: size.width, y: 0))
            let bottomLeft  = map.gps(withMapPt: CGPoint(x: 0, y: size.height))
            let bottomRight = map.gps(withMapPt: CGPoint(x: size.width, y: size.height))
            let polygon = MKPolygon(coordinates: [topLeft, bottomLeft, bottomRight, topRight], count: 4)
            self.appleMapView.addOverlay(polygon)
            self.meridianMapOverlay = polygon
            
            // Figure out a reasonable zoom span that will trigger our transition into the Meridian map
            self.latitudeSpanTrigger = max((abs(topLeft.latitude - bottomRight.latitude)), abs(topRight.latitude - bottomLeft.latitude))
            self.longitudeSpanTrigger = max((abs(topLeft.longitude - bottomRight.longitude)), abs(topRight.longitude - bottomLeft.longitude))
            
        }) { (error) in
            print("Error loading Meridian Map: \(error.localizedDescription)")
        }
    }
}

extension MSAppleMapViewController {
    @objc func zoomTapped(_ sender: UIBarButtonItem)  {
        if let map = meridianMap {
            let center = map.gps(withMapPt: CGPoint(x: map.size.width / 2.0 , y: map.size.height / 2.0))
            let camera = MKMapCamera(lookingAtCenter: center, fromEyeCoordinate: center, eyeAltitude: 3000)
            appleMapView.setCamera(camera, animated: true)
        }
    }
}

extension MSAppleMapViewController: MKMapViewDelegate {
    func mapView(_ mapView: MKMapView, rendererFor overlay: MKOverlay) -> MKOverlayRenderer {
        guard let polygon = overlay as? MKPolygon else {
            return MKOverlayRenderer(overlay: overlay) }
        let polygonRenderer = MKPolygonRenderer(polygon: polygon)
        polygonRenderer.strokeColor = #colorLiteral(red: 0, green: 0.7958288789, blue: 0.9350526333, alpha: 1)
        polygonRenderer.fillColor = #colorLiteral(red: 0, green: 0.7958288789, blue: 0.9350526333, alpha: 1).withAlphaComponent(0.75)
        polygonRenderer.alpha = 0.5
        return polygonRenderer
}

    func mapView(_ mapView: MKMapView, regionDidChangeAnimated animated: Bool) {
        if mapView.region.span.latitudeDelta < latitudeSpanTrigger || mapView.region.span.longitudeDelta < longitudeSpanTrigger {
            if let map = meridianMap, let overlay = meridianMapOverlay {
                
                // If we've zoomed in far enough and within the Meridian overlay, fade to the Meridian map
                let renderer = MKPolygonRenderer(overlay: overlay)
                if renderer.path.contains(renderer.point(for: MKMapPoint(mapView.centerCoordinate))) {
                    guard let mapViewController = DismissAtZoomMRMapViewController(editorKey: map.key) else {
                        print("Unable to create DismissAtZoomMRMapViewController")
                        return
                    }
                    
                    let currentHeading = mapView.camera.heading.magnitude
                    mapViewController.initialNorthOffset = currentHeading
                    
                    let mapNavigationController = UINavigationController(rootViewController: mapViewController)
                    mapNavigationController.modalTransitionStyle = .crossDissolve
                    present(mapNavigationController, animated: true, completion: nil)
                }
            }
        }
    }
}

extension MSAppleMapViewController: MSExampleViewController {
    static func exampleTitle() -> String {
        return "Apple to Meridian Transition"
    }
    
    static func exampleDetail() -> String {
        return """
        Switch between Apple Maps and Meridian Maps based on respective zoom level
        
        • Zoom button moves Apple map above Meridian location
        """
    }
}


